from fastapi import APIRouter
from fastapi.responses import StreamingResponse

from aiconsole.aic_types import ChatWithAgentAndMaterials, ExecutionModeContext
from aiconsole.agents.agents import agents
from aiconsole.utils.connection_manager import ErrorWSMessage, connection_manager

import logging

router = APIRouter()

log = logging.getLogger(__name__)


@router.post("/execute")
async def execute(chat: ChatWithAgentAndMaterials) -> StreamingResponse:
    agent = agents.agents[chat.agent_id]

    context = ExecutionModeContext(
        messages=chat.messages,
        agent=agent,
        relevant_materials=chat.relevant_materials,
        gpt_mode=agent.gpt_mode,
    )

    async def async_wrapper():
        try:
            async for item in agent.execution_mode(context):
                yield item
        except Exception as e:
            await connection_manager.send_message(chat.id, ErrorWSMessage(error=str(e)))
            raise e

    return StreamingResponse(
        async_wrapper(), media_type="text/event-stream"
    )
    
