import traceback
from fastapi import APIRouter
from fastapi.responses import StreamingResponse
from pydantic import BaseModel

from aiconsole.aic_types import Chat
from aiconsole.code_interpreters.get_code_interpreter import get_code_interpreter
from aiconsole.utils.connection_manager import ErrorWSMessage, connection_manager

import logging

router = APIRouter()

log = logging.getLogger(__name__)

class CodeToRun(BaseModel):
    language: str
    code: str

@router.post("/chats/{chat_id}/run")
async def execute(chat_id: str, codeToRun: CodeToRun) -> StreamingResponse:
    async def async_wrapper():
        try:
            log.debug("Running code: %s", codeToRun.code)

            try:
                for chunk in get_code_interpreter(codeToRun.language).run(codeToRun.code):
                    yield chunk
            except:
                await connection_manager.send_message(chat_id, ErrorWSMessage(error=traceback.format_exc().strip()))
                yield traceback.format_exc().strip()
        except Exception as e:
            await connection_manager.send_message(chat_id, ErrorWSMessage(error=str(e)))
            raise e

    return StreamingResponse(
        async_wrapper(), media_type="text/event-stream"
    )
    
