from pydantic import BaseModel
from typing import Optional, Any
import logging
from fastapi import APIRouter, WebSocket, WebSocketDisconnect

from aiconsole.utils.connection_manager import connection_manager

router = APIRouter()

log = logging.getLogger(__name__)

class Message(BaseModel):
    type: str
    detail: Any
    error: Optional[str]

@router.websocket("/ws/{chat_id}")
async def websocket_endpoint(websocket: WebSocket, chat_id: str):
    await connection_manager.connect(websocket, chat_id)

    try: 
        while True:
            json_data = await websocket.receive_text()
            message = Message.model_validate_json(json_data)
            # Handle the message accordingly
            # For example, when there is an error:
            if message.error:
                error_message = Message(type="error", detail={}, error="An error occured.")
                await websocket.send_text(error_message.json())
    except WebSocketDisconnect:
        connection_manager.disconnect(websocket, chat_id)
        # You could further send a message to the chat here, if needed
