import asyncio
import logging
from typing import AsyncGenerator

from aiconsole.aic_types import ExecutionModeContext
from aiconsole.execution_modes.messsage_from_llm import messsage_from_llm
from aiconsole.execution_modes.setup_openai_coding_llm import coding_llm
from aiconsole.gpt.create_full_prompt_from_sections import create_full_prompt_from_sections
from aiconsole.gpt.gpt_types import CLEAR_STR
from aiconsole.utils.convert_messages import convert_messages
from aiconsole.utils.get_user_info_string import get_user_info_string
import litellm

log = logging.getLogger(__name__)

litellm.set_verbose = False
model = "gpt-4"

temperature = 0

context_window = None
max_tokens = None
api_base = None
api_key = None
max_budget = None

async def execution_mode_interpreter(
    context: ExecutionModeContext,
) -> AsyncGenerator[str, None]:
    global llm
    
    # These are set directly on LiteLLM
    if max_budget:
        litellm.max_budget = max_budget

    system_message = create_full_prompt_from_sections(
        intro=context.agent.system,
        sections=context.relevant_materials,
        outro='\n\n---\n\n'
    )

    system_message += "\n\n" + get_user_info_string()

    log.debug(f"System message:\n{system_message}")

    in_code = False
    in_code_output = False

    messages = [
        {"role": "system", "content": system_message},
        *(message.model_dump() for message in convert_messages(context.messages))
    ]

    coding_llm_response = coding_llm(model, max_tokens, temperature, api_key, api_base, messages)
    for chunk in messsage_from_llm(coding_llm_response):
        log.debug(chunk)

        try:
            if chunk == CLEAR_STR:
                yield f"<<<< CLEAR >>>>"
                in_code = False
                in_code_output = False
            else:
                if not chunk.get("code", "") and not chunk.get("language", ""):
                    if in_code:
                        yield f"<<<< END CODE >>>>"
                        in_code = False

                if chunk.get("language", ""):
                    if in_code_output:
                        raise Exception("Language inside output")
                    if not in_code:
                        yield f'<<<< START CODE ({str(chunk.get("language", ""))}) >>>>'
                    in_code = True

                if chunk.get("code", ""):
                    if in_code_output:
                        raise Exception("Code inside output")

                    if not in_code:
                        raise Exception("Code without language")
                    yield str(chunk.get("code", ""))

                if chunk.get("message", ""):
                    if in_code:
                        yield f"<<<< END CODE >>>>"
                    in_code = False
                    in_code_output = False
                    yield str(chunk.get("message", ""))

            await asyncio.sleep(0)
        except asyncio.CancelledError:
            log.warning("Cancelled execution_mode_interpreter")
            break
