import decimal
from typing import Literal, Optional

from pydantic import BaseModel
from typing_extensions import TypedDict

GPTRole = Literal["user", "assistant", "system", "function"]

CLEAR_STR = "<<<< CLEAR >>>>"
CLEAR_STR_TYPE = Literal["<<<< CLEAR >>>>"]

class GPTMessage(BaseModel):
    role: GPTRole
    content: str
    function_call: Optional[object] = None
    name: Optional[str] = None

    def model_dump(self):
        # Don't include None values, call to super to avoid recursion
        return {k: v for k, v in super().model_dump().items() if v is not None}


class EnforcedFunctionCall(TypedDict):
    name: str


class GPTFunctionCallInvocation(TypedDict):
    name: str
    arguments: str


class GPTResult(BaseModel):
    content: str
    function_call: Optional[GPTFunctionCallInvocation]
    cost: decimal.Decimal
